@@ -136,8 +136,8 @@ class Solver(object):
         Dict of list of var names to record
     _norm0: float
         Normalization factor
-    _solver_info : SolverInfo
-        A stack-like object shared by all Solvers in the model.
+    _problem_meta : dict
+        Problem level metadata.
     """
 
     # Object to store some formatting for iprint that is shared across all solvers.
@@ -157,7 +157,7 @@ class Solver(object):
         self._vec_names = None
         self._mode = 'fwd'
         self._iter_count = 0
-        self._solver_info = None
+        self._problem_meta = None
 
         # Solver options
         self.options = OptionsDictionary(parent_name=self.msginfo)
@@ -186,7 +186,11 @@ class Solver(object):
                                        desc='Set to True to record outputs at the solver level')
         self.recording_options.declare('record_solver_residuals', types=bool, default=False,
                                        desc='Set to True to record residuals at the solver level')
-        self.recording_options.declare('record_metadata', types=bool, desc='Record metadata',
+        self.recording_options.declare('record_metadata', types=bool, desc='Deprecated. Recording '
+                                       'of metadata will always be done',
+                                       deprecation="The recording option, record_metadata, on "
+                                       "Solver is "
+                                       "deprecated. Recording of metadata will always be done",
                                        default=True)
         self.recording_options.declare('includes', types=list, default=['*'],
                                        desc="Patterns for variables to include in recording. \
@@ -228,6 +232,20 @@ class Solver(object):
             return type(self).__name__
         return '{} in {}'.format(type(self).__name__, self._system().msginfo)
 
+    @property
+    def _recording_iter(self):
+        if self._problem_meta is None:
+            raise RuntimeError(f"{self.msginfo}: Can't access recording_iter because "
+                               "_setup_solvers has not been called.")
+        return self._problem_meta['recording_iter']
+
+    @property
+    def _solver_info(self):
+        if self._problem_meta is None:
+            raise RuntimeError(f"{self.msginfo}: Can't access solver_info because _setup_solvers "
+                               "has not been called.")
+        return self._problem_meta['solver_info']
+
     def _assembled_jac_solver_iter(self):
         """
         Return an empty generator of lin solvers using assembled jacs.
@@ -270,8 +288,7 @@ class Solver(object):
         """
         self._system = weakref.ref(system)
         self._depth = depth
-        self._solver_info = system._solver_info
-        self._recording_iter = system._recording_iter
+        self._problem_meta = system._problem_meta
 
         if system.pathname:
             parent_name = self.msginfo
@@ -385,10 +402,10 @@ class Solver(object):
                 # be wrapped in the with for stack purposes, so we locally assign  norm & norm0
                 # into the class.
                 rec.abs = norm
+                if norm0 == 0:
+                    norm0 = 1
                 rec.rel = norm / norm0
 
-            if norm0 == 0:
-                norm0 = 1
             self._mpi_print(self._iter_count, norm, norm / norm0)
 
         system = self._system()
@@ -543,15 +560,11 @@ class Solver(object):
             return
 
         from openmdao.core.group import Group
-        if (isinstance(s, Group) and s._has_distrib_vars) or (isinstance(s, Component) and
-                                                              s.options['distributed']):
-            raise RuntimeError("%s has a %s solver and contains a distributed system." %
-                               (s.msginfo, type(self).__name__))
-
-        if not (np.all(s._var_sizes['nonlinear']['output']) and
-                np.all(s._var_sizes['nonlinear']['input'])):
-            raise RuntimeError("%s has a %s solver and contains remote variables." %
-                               (s.msginfo, type(self).__name__))
+        if (isinstance(s, Group) and (s._has_distrib_vars or s._contains_parallel_group)) or \
+           (isinstance(s, Component) and s.options['distributed']):
+            msg = "{} linear solver in {} cannot be used in or above a ParallelGroup or a " + \
+                "distributed component."
+            raise RuntimeError(msg.format(type(self).__name__, s.msginfo))
 
 
 class NonlinearSolver(Solver):
@@ -676,11 +689,10 @@ class NonlinearSolver(Solver):
         Perform a Gauss-Seidel iteration over this Solver's subsystems.
         """
         system = self._system()
-        for isub, (subsys, local)in enumerate(system._all_subsystem_iter()):
+        for isub, subsys in enumerate(system._subsystems_allprocs):
             system._transfer('nonlinear', 'fwd', isub)
 
-            if local:
-
+            if subsys._is_local:
                 try:
                     subsys._solve_nonlinear()
                 except AnalysisError as err:
@@ -688,8 +700,6 @@ class NonlinearSolver(Solver):
                             self.options['reraise_child_analysiserror']:
                         raise err
 
-            system._check_child_reconf(subsys)
-
 
 class LinearSolver(Solver):
     """
